using System;
using static System.Console;
using static System.Math;

class Board {
  public int[] square;
  int moves = 0;
  
  public Board(int n) { square = new int[n]; }
  
  public int size => square.Length;
  
  // Return 1 if X wins, -1 if O wins, 0 if a draw.
  public int? result() {
    if (moves < size) return null;
    
    int streak = 0;
    int longest_x = 0, longest_o = 0;
    
    for (int i = 0 ; i < size ; ++i) {
      if (i > 0 && square[i] == square[i - 1])
        streak += 1;
      else
        streak = 1;
        
      if (square[i] == 1)
        longest_x = Max(longest_x, streak);
      else longest_o = Max(longest_o, streak);
    }
    return Sign(longest_x - longest_o);
  }
    
  public bool move(int player, int i) {
    if (square[i] == 0) {
      square[i] = player;
      ++moves;
      return true;
    } else return false;
  }
  
  public void unmove(int i) {
    square[i] = 0;
    --moves;
  }
}

class Minimax {
  // Return the best possible score and best move for the current player.
  public static int minimax(Board b, int player, int alpha, int beta, out int move) {
    move = -1;
    if (b.result() is int r)
      return r;  // game is done
    
    int m = player == 1 ? int.MinValue : int.MaxValue;
    for (int i = 0 ; i < b.size ; ++i)
      if (b.move(player, i)) {
        int n = minimax(b, 3 - player, alpha, beta, out int dummy);
        b.unmove(i);
        
        if (player == 1) {
          if (n > m) {
            m = n;
            alpha = Max(alpha, n);
            move = i;  // the best move so far
          }
          if (m == 1 || m >= beta)
            break;
        } else { // player 2
          if (n < m) {
            m = n;
            beta = Min(beta, n);
            move = i;  // the best move so far
          }
          if (m == -1 || m <= alpha)
            break;
        }
      }  // end if
    return m;
  }
  
  public static int minimax(Board b, int player, out int move) =>
    minimax(b, player, int.MinValue, int.MaxValue, out move);
}

class Game {
  static void eval() {
    for (int size = 1 ; size <= 11 ; ++size) {
      DateTime start = DateTime.Now;
      int result = Minimax.minimax(new Board(size), 1, out int first);
      int ms = (int) (DateTime.Now - start).TotalMilliseconds;
      WriteLine($"{size} -> {result} (first move {first}) ({ms} ms)");
    }
  }
  
  const string Symbols = " XO";

  static void draw(Board b) {
    Clear();
    
    for (int i = 0 ; i < b.size ; ++i)
      Write(Symbols[b.square[i]] + " ");
    WriteLine();
    for (int i = 0 ; i < b.size ; ++i)
      Write(i + " ");
    WriteLine();
  }
  
  static void Main() {
    Write("board size? ");
    int size = int.Parse(ReadLine());
    Write("are you X or O? ");
    int human;
    switch (ReadLine().ToLower()) {
      case "x": human = 1; break;
      case "o": human = 2; break;
      default: throw new Exception("bad response");
    }
    
    Board b = new Board(size);
    int player = 1;
    while (true) {
      int move;
      
      if (player == human) {
        draw(b);
        move = int.Parse(ReadLine());
      } else Minimax.minimax(b, player, out move);
      
      if (!b.move(player, move)) throw new Exception("bad move");
      
      if (b.result() is int winner) {
        draw(b);
        WriteLine(winner > 0 ? $"{Symbols[winner]} wins" : "It's a draw");
        return;
      }
      
      player = 3 - player;
    }
  }
}
